<?php
/*
Plugin Name: Piwishadow
Author: Binaryworld
Author URI: http://binaryworld.hd.free.fr/
*/
if (!defined('PIWISHADOW_PATH')) die('Hacking attempt!');

class Cursor {
	private $_config;
	private $_section = null;

	public function  __construct(&$config, $section) {
		$this->_config = &$config;
		$this->_section = $section;
	}
	
	public function moveTo($section) {
		$this->_section = $section;
	}
	
	public function get($param) {
		return $this->_config[$this->_section][$param]; 
	}
	
	public function set($param, $value) {
		$this->_config[$this->_section][$param] = $value;
	}
	
	public function getBool($param) {
		return Config::str2bool( $this->_config[$this->_section][$param] );
	}
}


class Config {
	public static $DEFAULT_CONFIG_FILENAME = 'default_config.ini.php';
	public static $CONFIG_FILENAME = 'config.ini.php';
	private $_config;
	
	public static function str2bool($val, $trueValues = array('yes', 'true', '1'), $lowerCase = true) {
		if (is_string($val)) {
			return (in_array( ($lowerCase ? strtolower($val) : $val), $trueValues) );
		} else {
			return (boolean) $val;
		}
	}
	
	// @from http://www.php.net/manual/en/function.parse-ini-file.php#78060
	public static function read($path, &$r) {
		$null = "";
		$r=$null;
		$first_char = "";
		$sec=$null;
		$comment_chars=";#";
		$num_comments = "0";

		//Read to end of file with the newlines still attached into $f
		$f = @file($path);
		if ($f === false) {
			return false;
		}
		// Process all lines from 0 to count($f)
		for ($i=0; $i<@count($f); $i++)
		{
			$w=@trim($f[$i]);
			$first_char = @substr($w,0,1);
			if ($w)
			{
				if ((@substr($w,0,1)=="[") and (@substr($w,-1,1))=="]") {
					$sec=@substr($w,1,@strlen($w)-2);
					$num_comments = 0;
					$num_newline = 0;
				}
				else if ((stristr($comment_chars, $first_char) == true)) {
					$r[$sec]["Comment_".$num_comments]=$w;
					$num_comments = $num_comments +1;
				}
				else {
					// Look for the = char to allow us to split the section into key and value
					$w=@explode("=",$w);
					$k=@trim($w[0]);
					unset($w[0]);
					$v=@trim(@implode("=",$w));
					// look for the new lines
					if ((@substr($v,0,1)=="\"") and (@substr($v,-1,1)=="\"")) {
						$v=@substr($v,1,@strlen($v)-2);
					}

					if (empty($sec))
						$r[$k] = $v;
					else
						$r[$sec][$k] = $v;
				}
			}
		}
		return true;
	}
	
	// @from http://www.php.net/manual/en/function.parse-ini-file.php#78060
	public static function write($path, $config) {
		$content = "";
		
		foreach ($config as $key=>$elem) {
			if (is_array($elem)) {
				if ($key != '') {
					$content .= "[".$key."]\r\n";
				}
				 
				foreach ($elem as $key2=>$elem2) {
					if ((substr_compare($key2,'Comment_',0,8) == 0) && (substr_compare($elem2,';',0,1) == 0)) {
						$content .= $elem2."\r\n";
					}
					else {
						$content .= $key2." = ".$elem2."\r\n";
					}
				}
			}
			else {
				$content .= $key." = ".$elem."\r\n";
			}
		}
		
		if (!$handle = fopen($path, 'w')) {
			return false;
		}
		if (!fwrite($handle, $content)) {
			return false;
		}
		fclose($handle);
		return true;		
	}
	
	public static function updateConfig($config, $with) {
		foreach ($with AS $k => $v) {
			// do not update the params or sections which are not defined in $config
			if (array_key_exists($k, $config)) {
				if (is_array($v))
					$config[$k] = self::updateConfig($config[$k], $with[$k]);
				// do not update the comments
				else if (substr_compare($k,'Comment_',0,8) != 0)
					$config[$k] = $v;
			}
		}
		return $config;
	}
	
	public static function createDefaultConfig($path = null) {
		$path = (empty($path)) ? dirname(__FILE__) .DS. self::$DEFAULT_CONFIG_FILENAME : $path;
		if (file_exists($path)) {
			Config::read($path, $config);
			return new Config( $config );
		}
		return false;
	}
	
	public function  __construct($config) {
		$this->_config = $config;
	}
	
	public function cursor($section) {
		return new Cursor($this->_config, $section);
	}
	
	// access ini values at format "section_name.property", 
	// for example $myconf->property("system.name") returns a property "name" in section "system"
	// @from http://www.php.net/manual/en/function.parse-ini-file.php#54737
	function property($name) {
		if(strpos($name, ".")) {
			list($section_name, $property) = explode(".", $name);
			$section =& $this->properties[$section_name];
			$name = $property;
		} else {
			$section =& $properties;
		}
	
		if(is_array($section) && isset($section[$name])) {
			return $section[$name];
		}
		return false;
	}
	
	public function update($with) {
		$this->_config = self::updateConfig($this->_config, $with);
	}
	
	public function save($path = null) {
		$path = (empty($path)) ? dirname(__FILE__) .DS. self::$CONFIG_FILENAME : $path;
		return self::write($path, $this->_config);
	}
	
	public function getBool($param, $section = null) {
		$value = ($section) ? $this->_config[$section][$param] : $this->_config[$param];
		return self::str2bool( $value );
	}
	
	public function get($param, $section = null) {
		return ($section) ? $this->_config[$section][$param] : $this->_config[$param];
	}
	
	public function set($param, $section, $value) {
		if (empty($section)) 
			$this->_config[$param] = $value;
		else
			$this->_config[$section][$param] = $value;
	}
	
	public function section($section) {
		return $this->_config[$section];
	}
	
}

?>
